@@ -5,7 +5,6 @@
 import os
 from typing import Dict, Union, Any
 
-import urllib3
 from tqdm import tqdm
 
 from qcloud_sdk.cos.utils import verify_file_crc64, remove_if_exists, get_file_size
@@ -15,7 +14,7 @@ class CosObjectAPIMixin(object):
     """
     对象API
     """
-    def head_object(self, object_key: str, bucket=None, region=None, appid=None):
+    def head_object(self, object_key: str, bucket=None, region=None, appid=None) -> dict:
         """
 
         :param object_key: 对象键
@@ -29,7 +28,7 @@ class CosObjectAPIMixin(object):
         headers = {}
         response = self.request_cos_bucket_api(method='HEAD', path=f'/{object_key}', query_params=query_params, headers=headers,
                                                bucket=bucket, region=region, appid=appid)
-        return dict(response.headers)
+        return response
 
     def get_object(self, object_key: str, bucket=None, region=None, appid=None,
                    range_begin=None, range_end=None):
@@ -61,8 +60,33 @@ class CosObjectAPIMixin(object):
                                                bucket=bucket, region=region, appid=appid, stream=True)
         return response
 
+    def post_object(self):
+        pass
+
+    def put_object(self, object_key, data=None, headers=None, bucket=None, region=None, appid=None):
+        response = self.request_cos_bucket_api('PUT', path=f'/{object_key}', query_params=None, data=data, headers=headers, bucket=bucket, region=region, appid=appid)
+        return response
+
+    def delete_object(self, object_key, headers=None, bucket=None, region=None, appid=None):
+        response = self.request_cos_bucket_api('DELETE', path=f'/{object_key}', query_params=None, headers=headers, bucket=bucket, region=region, appid=appid)
+        return response
+
 
 class CosObjectIntegratedAPIMixin(object):
+    def exists_object(self, object_key, bucket=None, region=None, appid=None) -> bool:
+        """
+        (Integrated API) 对象是否存在
+        :return:
+        """
+        response = self.head_object(object_key=object_key, region=region, appid=appid)
+        if response.status_code == 200:
+            return True
+        elif response.status_code == 404:
+            return False
+        else:
+            # TODO：进一步验证不同情况下的状态码以后规范抛出值
+            raise ValueError('状态码异常')
+
     def download_object_to_file(self, object_key, file_path, bucket=None, region=None, appid=None,
                                 request_chunk_size=1024*1024*20, file_chunk_size=1024*1024,
                                 remove_existed_tmp_file: bool = False,
@@ -96,9 +120,9 @@ class CosObjectIntegratedAPIMixin(object):
         existed_tmp_file_size = get_file_size(tmp_file_path)
 
         # 获取对象元数据
-        headers = self.head_object(object_key=object_key, bucket=bucket, region=region, appid=appid)
+        response = self.head_object(object_key=object_key, bucket=bucket, region=region, appid=appid)
         # 获取对象长度
-        content_length = int(headers['Content-Length'])
+        content_length = response.content_length
 
         # 分块下载文件
         request_ranges = [(i, min(i-1+request_chunk_size, content_length)) for i in range(existed_tmp_file_size, content_length, request_chunk_size)]
@@ -110,7 +134,7 @@ class CosObjectIntegratedAPIMixin(object):
 
         # CRC64校验
         # TODO：校验结果写入日志，包括CRC64的值、校验结果是否正确。
-        if not verify_file_crc64(int(headers['x-cos-hash-crc64ecma']), tmp_file_path, file_chunk_size):
+        if not verify_file_crc64(response.hash_crc64ecma, tmp_file_path, file_chunk_size):
             # 校验失败文件支持自动清空，以方便捕获异常后重新下载。
             os.remove(tmp_file_path) if remove_unverified_file else None
             # TODO：换成自定义异常类，以方便被上级程序捕获。
